<?php

namespace App\Commands;

use App\Commands\Concerns\ClonesProjects;
use App\Commands\Concerns\SetProjectStorage;
use App\Git\AccessToken;
use App\Git\ProjectPicker;
use App\Installation\InstructionManual;
use App\Projects\ProjectBase;
use App\Projects\ProjectName;
use App\Tasks\ApplySymlinks;
use App\Tasks\CopyProjectTask;
use App\Tasks\InitialiseProject;
use LaravelZero\Framework\Commands\Command;

class InstallProject extends Command
{
    use ClonesProjects, SetProjectStorage;

    /**
     * The signature of the command.
     *
     * @var string
     */
    protected $signature = 'install {name? : Name of the application you want to install (required)} {-f|--folder= : The relative folder to create the application in (optional)}';

    /**
     * The description of the command.
     *
     * @var string
     */
    protected $description = 'Install an existing project.';

    /**
     * Execute the console command.
     *
     * @return mixed
     */
    public function handle()
    {
        $projectName = $this->getProjectName();

        if ($projectName === null) {
            $this->error('Project not selected');

            return;
        }

        ProjectName::save($projectName);
        $url = 'git@git.drp.digital:' . $projectName . '.git';
        $project = new ProjectBase($projectName, $projectName, $url);
        $destinationFolder = $this->option('folder') ?? $projectName;

        $this->cloneProject($project);
        $this->task('Installing ' . $project->label, new CopyProjectTask($project, $destinationFolder));
        $this->title('Running setup commands');

        $this->createProjectDisk($destinationFolder);

        InitialiseProject::run($project->label, $this, $this->app[InstructionManual::class]);
        $this->task('Apply symlinks', new ApplySymlinks($project, $destinationFolder));
        ProjectName::forget();
    }

    private function getProjectName(): string
    {
        $projectName = $this->argument('name');

        if ($projectName === null) {
            return $this->selectProject();
        }

        return $projectName;
    }

    private function selectProject(): ?string
    {
        $this->getOutput()->block([
            'No project specified. Fetching all of your projects from GitLab.',
        ], null, 'question', ' ', true);

        $accessToken = AccessToken::retrieve(function () {
            $this->info('No existing access token found, please provide your GitLab credentials.');

            $username = $this->ask('What is your GitLab username?');
            $password = $this->secret('What is your GitLab password?');

            return [$username, $password];
        });

        if ($accessToken === null) {
            return null;
        }

        return (new ProjectPicker($this))->pick($accessToken);
    }
}
